#!/usr/bin/env python
#	"$Name:  $";
#	"$Header:  $";
#=============================================================================
#
# file :        EvrAttr.py
#
# description : Python source for the EvrAttr and its commands. 
#                The class is derived from Device. It represents the
#                CORBA servant object which will be accessed from the
#                network. All commands which can be executed on the
#                EvrAttr are implemented in this file.
#
# project :     TANGO Device Server
#
# $Author:  $
#
# $Revision:  $
#
# $Log:  $
#
# copyleft :    European Synchrotron Radiation Facility
#               BP 220, Grenoble 38043
#               FRANCE
#
#=============================================================================
#  		This file is generated by POGO
#	(Program Obviously used to Generate tango Object)
#
#         (c) - Software Engineering Group - ESRF
#=============================================================================
#


import PyTango
import sys
import time

#==================================================================
#   EvrAttr Class Description:
#
#         Export attributes for the event system.
#
#==================================================================


class EvrAttr(PyTango.Device_4Impl):

#--------- Add you global variables here --------------------------

#------------------------------------------------------------------
#	Device constructor
#------------------------------------------------------------------
	def __init__(self,cl, name):
		PyTango.Device_4Impl.__init__(self,cl,name)
                self.ChannelLabel = ""
		EvrAttr.init_device(self)

#------------------------------------------------------------------
#	Device destructor
#------------------------------------------------------------------
	def delete_device(self):
		#print "[Device delete_device method] for device",self.get_name()
		return


#------------------------------------------------------------------
#	Device initialization
#------------------------------------------------------------------
	def init_device(self):
		#print "In ", self.get_name(), "::init_device()"
		self.set_state(PyTango.DevState.ON)
		self.TimeConversionEnabled = False
		self.get_device_properties(self.get_device_class())
		self.evr_proxy = PyTango.DeviceProxy(self.Evrdevice)
		self.lastUpdateTime = 0


#------------------------------------------------------------------
#	Always excuted hook method
#------------------------------------------------------------------
	def always_executed_hook(self):
		#print "In ", self.get_name(), "::always_excuted_hook()"
		if (time.time() - self.lastUpdateTime) < (self.UpdatePeriod_ms / 1000.):
		  return
		aux = self.evr_proxy.command_inout("GetOneTimePulseConfig",self.Channel)
		if (self.TimeConversionEnabled):
			self.delay = float((aux[0] * 12.675) /1000.0)
			self.width = float((aux[1] * 12.675)/1000.0) 
		else:
			self.delay = float(aux[0])
			self.width = float(aux[1]) 
		self.polarity = (aux[2] == 1)
		self.lastUpdateTime = time.time()
		
#==================================================================
#
#	EvrAttr read/write attribute methods
#
#==================================================================
#------------------------------------------------------------------
#	Read Attribute Hardware
#------------------------------------------------------------------
	def read_attr_hardware(self,data):
		#print "In ", self.get_name(), "::read_attr_hardware()"
		return


#------------------------------------------------------------------
#	Read Delay attribute
#------------------------------------------------------------------
	def read_Delay(self, attr):
		#print "In ", self.get_name(), "::read_Delay()"
		
		#	Add your own code here
		attr.set_value(self.delay)


#------------------------------------------------------------------
#	Write Delay attribute
#------------------------------------------------------------------
	def write_Delay(self, attr):
		#print "In ", self.get_name(), "::write_Delay()"
		data = attr.get_write_value()
		self.delay = data
		if (self.TimeConversionEnabled):
			self.evr_proxy.command_inout("OneTimePulseConfig",[self.Channel, int(self.delay*1000/12.675),int(self.width*1000/12.675),int(self.polarity)])
		else:
			self.evr_proxy.command_inout("OneTimePulseConfig",[self.Channel, int(self.delay),int(self.width),int(self.polarity)])
		self.lastUpdateTime = 0

#------------------------------------------------------------------
#	Read Width attribute
#------------------------------------------------------------------
	def read_Width(self, attr):
		#print "In ", self.get_name(), "::read_Width()"
		
		#	Add your own code here
		attr.set_value(self.width)


#------------------------------------------------------------------
#	Write Width attribute
#------------------------------------------------------------------
	def write_Width(self, attr):
		#print "In ", self.get_name(), "::write_Width()"
		data = attr.get_write_value()
                #print "Attribute value = ", data
		self.width = data
		#	Add your own code here
		if (self.TimeConversionEnabled):
			self.evr_proxy.command_inout("OneTimePulseConfig",[self.Channel, int(self.delay*1000/12.675),int(self.width*1000/12.675),int(self.polarity)])
		else:
			self.evr_proxy.command_inout("OneTimePulseConfig",[self.Channel, int(self.delay),int(self.width),int(self.polarity)])
		self.lastUpdateTime = 0

#------------------------------------------------------------------
#	Read Polarity attribute
#------------------------------------------------------------------
	def read_Polarity(self, attr):
		#print "In ", self.get_name(), "::read_Polarity()"
		
		#	Add your own code here
		attr.set_value(self.polarity)


#------------------------------------------------------------------
#	Write Polarity attribute
#------------------------------------------------------------------
	def write_Polarity(self, attr):
		#print "In ", self.get_name(), "::write_Polarity()"
		data = attr.get_write_value()
		#print "Attribute value = ", data
		self.polarity = data
		#	Add your own code here
		if (self.TimeConversionEnabled):
			self.evr_proxy.command_inout("OneTimePulseConfig",[self.Channel, int(self.delay*1000/12.675),int(self.width*1000/12.675),int(self.polarity)])
		else:
			self.evr_proxy.command_inout("OneTimePulseConfig",[self.Channel, int(self.delay),int(self.width),int(self.polarity)])
		self.lastUpdateTime = 0

#------------------------------------------------------------------
#	Read ChannelLabel attribute
#------------------------------------------------------------------
	def read_ChannelLabel(self, attr):
		#print "In ", self.get_name(), "::read_Polarity()"
		
		#	Add your own code here
		attr.set_value(self.ChannelLabel)


#------------------------------------------------------------------
#	Write ChannelLabel attribute
#------------------------------------------------------------------
	def write_ChannelLabel(self, attr):
		#print "In ", self.get_name(), "::write_Polarity()"
		data = attr.get_write_value()
		#print "Attribute value = ", data
		self.ChannelLabel = data
		#Add your own code here

#------------------------------------------------------------------
#	Read EnablePulseTrain attribute
#------------------------------------------------------------------
	def read_EnablePulseTrain(self, attr):
		#print "In ", self.get_name(), "::read_Polarity()"
		
		#	Add your own code here
		aux = self.evr_proxy.command_inout("GetOneTimePulseEnable",self.Channel)
		attr.set_value(aux)


#------------------------------------------------------------------
#	Write EnablePulseTrain attribute
#------------------------------------------------------------------
	def write_EnablePulseTrain(self, attr):
		#print "In ", self.get_name(), "::write_Polarity()"
		data = attr.get_write_value()
		#print "Attribute value = ", data
                enabling = data
		#Add your own code here
                if (enabling):
		  self.evr_proxy.command_inout("OneTimePulseEnable",self.Channel)
		else:
		  self.evr_proxy.command_inout("OneTimePulseDisable",self.Channel)
		self.lastUpdateTime = 0

#==================================================================
#
#	EvrAttr command methods
#
#==================================================================

#==================================================================
#
#	EvrAttrClass class definition
#
#==================================================================
class EvrAttrClass(PyTango.DeviceClass):

	#	Class Properties
	class_property_list = {
		}


	#	Device Properties
	device_property_list = {
		'Evrdevice':
			[PyTango.DevString,
			"",
			[] ],
		'Channel':
			[PyTango.DevLong,
			"",
			[] ],
		'TimeConversionEnabled':
			[PyTango.DevBoolean,
			"",
			[] ],
		'UpdatePeriod_ms':
			[PyTango.DevLong,
			"",
			[5000] ],
		}


	#	Command definitions
	cmd_list = {
		}


	#	Attribute definitions
	attr_list = {
		'Delay':
			[[PyTango.DevFloat,
			PyTango.SCALAR,
			PyTango.READ_WRITE],
			{
				'Memorized':"false",
			} ],
		'Width':
			[[PyTango.DevFloat,
			PyTango.SCALAR,
			PyTango.READ_WRITE],
			{
				'Memorized':"false",
			} ],
		'Polarity':
			[[PyTango.DevBoolean,
			PyTango.SCALAR,
			PyTango.READ_WRITE],
			{
				'Memorized':"false",
			} ],
		'ChannelLabel':
			[[PyTango.DevString,
			PyTango.SCALAR,
			PyTango.READ_WRITE],
			{
				'Memorized':"true",
			} ],
		'EnablePulseTrain':
			[[PyTango.DevBoolean,
			PyTango.SCALAR,
			PyTango.READ_WRITE],
			{
				'Memorized':"false",
			} ],
		}


#------------------------------------------------------------------
#	EvrAttrClass Constructor
#------------------------------------------------------------------
	def __init__(self, name):
		PyTango.DeviceClass.__init__(self, name)
		self.set_type(name);
		#print "In EvrAttrClass  constructor"

#==================================================================
#
#	EvrAttr class main method
#
#==================================================================
if __name__ == '__main__':
	try:
		py = PyTango.Util(sys.argv)
		py.add_TgClass(EvrAttrClass,EvrAttr,'EvrAttr')

		U = PyTango.Util.instance()
		U.server_init()
		U.server_run()

	except PyTango.DevFailed,e:
		print '-------> Received a DevFailed exception:',e
	except Exception,e:
		print '-------> An unforeseen exception occured....',e
